load("@ytt:data", "data")
load("@ytt:assert", "assert")
load("/helpers.star", "exactly_one", "one_or_none")

def validate():
	# kp_default_repository is always required
	data.values.kp_default_repository or assert.fail("missing kp_default_repository")

	exclusive_kp_default_repo_values = [(data.values.kp_default_repository_username and data.values.kp_default_repository_password),
		(data.values.kp_default_repository_secret.name and data.values.kp_default_repository_secret.namespace),
		data.values.kp_default_repository_aws_iam_role_arn]
	exactly_one(exclusive_kp_default_repo_values) or assert.fail("need exactly one of kp_default_repository_secret, kp_default_repository_username/password, or kp_default_repository_aws_iam_role_arn")

	# kp_default_repository creds and secret cannot both be configured
	if (data.values.kp_default_repository_secret.name or data.values.kp_default_repository_secret.name) and (data.values.kp_default_repository_username or data.values.kp_default_repository_password):
		assert.fail("only one of kp_default_repository_secret or kp_default_repository_username/password may be used")
	end

	# kp_default_repository_aws_iam_role_arn cannot be used with pull_from_kp_default_repo
	if data.values.pull_from_kp_default_repo and data.values.kp_default_repository_aws_iam_role_arn != "":
		assert.fail("kp_default_repository_aws_iam_role_arn and pull_from_kp_default_repo cannot both be set")
	end

	# validate no more than 1 source/image-pull config is set
	exclusive_pull_values = [data.values.pull_from_kp_default_repo, data.values.pull_secret_export, data.values.tbs_source_registry, data.values.tbs_source_secret.name]
	one_or_none(exclusive_pull_values) or assert.fail("only one of pull_from_kp_default_repo, pull_secret_export, tbs_source_registry*, tbs_source_secret can be used.")

	# if tbs_source_registry is used, creds must be set
	if data.values.tbs_source_registry != "":
		data.values.tbs_source_registry_username or assert.fail("missing tbs_source_registry_username")
		data.values.tbs_source_registry_password or assert.fail("missing tbs_source_registry_password")
	end

	# validate tanzunet username/password and secret are not both set
	if (data.values.tanzunet_secret.name or data.values.tanzunet_secret.namespace) and (data.values.tanzunet_username or data.values.tanzunet_password):
		assert.fail("only one of tanzunet_secret or tanzunet_username/password are allowed")
	end

	# require descriptor_name to be set when using dependency updater
	if (data.values.tanzunet_username or data.values.tanzunet_secret.name) and data.values.enable_automatic_dependency_updates and not data.values.descriptor_name:
		assert.fail("descriptor_name must be set when enable_automatic_dependency_updates is true. set enable_automatic_dependency_updates to false or set descriptor_name to 'full' or 'lite'")
	end

	# dependencies cannot be excluded when automatic updates is set
	if (data.values.enable_automatic_dependency_updates and data.values.exclude_dependencies):
		assert.fail("automatic updates is not possible when dependencies are excluded")
	end

	# require descriptor_name to be set when using dependency updater
	if (data.values.descriptor_name and not data.values.enable_automatic_dependency_updates):
		assert.fail("descriptor name can not be used if automatic updates is not enabled")
	end

	# require tanzunet creds with enable_automatic_dependency_updates
	if data.values.enable_automatic_dependency_updates and ((not data.values.tanzunet_secret.name) and (not data.values.tanzunet_username)):
		assert.fail("tanzunet credentials are required for automatic updates")
	end

	# validate kubernetes distribution
    kubernetes_distribution_values = ["", "openshift"]
    if data.values.kubernetes_distribution not in kubernetes_distribution_values:
      assert.fail("invalid kubernetes distribution provided. allowed values are {}".format(kubernetes_distribution_values))
    end

	# validate stack_configuration
	stack_configuration_values = ["default", "jammy-only"]
	if data.values.stack_configuration not in stack_configuration_values:
	  assert.fail("invalid stack_configuration provided. allowed values are {}".format(stack_configuration_values))
	end
end
